package com.bergerkiller.bukkit.nolagg;

import java.util.LinkedList;
import java.util.logging.Level;

import org.bukkit.Bukkit;

public class OperationScheduler extends Thread {
	
	private static LinkedList<ThreadOperation> added = new LinkedList<ThreadOperation>();
	private static LinkedList<ThreadOperation> operations = new LinkedList<ThreadOperation>();
	private static LinkedList<ThreadOperation> finished = new LinkedList<ThreadOperation>();
	private static boolean running = false;
	private static boolean ended = false;
	private static int endtaskid = -1;

	public static void start(ThreadOperation operation) {
		synchronized (added) {
			added.offer(operation);
		}
		operation.start();
	}
	
	private static void startAll() {
		synchronized (added) {
			ThreadOperation op;
			while ((op = added.poll()) != null) {
				try {
				    op.start();
				    synchronized (operations) {
				    	operations.offer(op);
				    }
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while starting a scheduled operation:");
					t.printStackTrace();
				}
			}
		}
	}
	private static void endAll() {
		synchronized (finished) {
			ThreadOperation op;
			while ((op = finished.poll()) != null) {
				try {
				    op.end();
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while finishing a scheduled operation:");
					t.printStackTrace();
				}
			}
		}
	}
	private static void run(ThreadOperation op) {
		op.run();
		synchronized (finished) {
			finished.offer(op);
		}
	}
	
	public static void init() {
		running = true;
		new OperationScheduler().start();
		endtaskid = Bukkit.getServer().getScheduler().scheduleSyncRepeatingTask(NoLagg.plugin, new Runnable() {
			public void run() {
				startAll();
				endAll();
			}
		}, 1, 1);
	}
	public static void deinit() {
		if (endtaskid != -1) {
			Bukkit.getServer().getScheduler().cancelTask(endtaskid);
		}
		running = false;
		//wait for it to be finished
		synchronized (operations) {
			if (!operations.isEmpty()) {
				NoLagg.log(Level.INFO, "Performing remaining scheduled operations (" + operations.size() + ") ...");
			}
			while (!ended) {
				try {
					Thread.sleep(10);
				} catch (InterruptedException ex) {}
			}
		}
		startAll();
		synchronized (operations) {
			for (ThreadOperation op : operations) {
				run(op);
			}
			operations.clear();
		}
		endAll();
	}
	
	public void run() {
		ThreadOperation operation;
		while (!this.isInterrupted() && running) {
			synchronized (operations) {
				operation = operations.poll();
			}
			if (operation == null) {
				try {
					Thread.sleep(300);
				} catch (InterruptedException ex) {}
			} else {
				try {
					run(operation);
				} catch (Throwable t) {
					NoLagg.log(Level.SEVERE, "An error occured while processing a scheduled operation:");
					t.printStackTrace();
				}
			}
		}
		ended = true;
	}

}
