package com.bergerkiller.bukkit.nolagg.bufferedtag;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.Map;

import net.minecraft.server.NBTBase;
import net.minecraft.server.NBTTagByte;
import net.minecraft.server.NBTTagDouble;
import net.minecraft.server.NBTTagEnd;
import net.minecraft.server.NBTTagFloat;
import net.minecraft.server.NBTTagInt;
import net.minecraft.server.NBTTagLong;
import net.minecraft.server.NBTTagShort;
import net.minecraft.server.NBTTagString;

public class NBTBaseBuffered {
	public static NBTBase readFrom(final Map<String, byte[]> buffers, DataInputStream stream) throws IOException {
		return readFrom(stream.readByte(), buffers, stream);
	}
	public static NBTBase readFrom(byte type, final Map<String, byte[]> buffers, DataInputStream stream) throws IOException {
        if (type == 0) {
            return new NBTTagEnd();
        } else {
            return readFrom(type, stream.readUTF(), buffers, stream);
        }
	}
	
	public static NBTBase readFrom(byte type, String name, final Map<String, byte[]> buffers, DataInputStream stream) throws IOException {
        NBTBase nbtbase;
        if (type == 10) {
        	nbtbase = NBTTagCompoundBuffered.readFrom(name, buffers, stream);
        } else if (type == 7) {
        	nbtbase = NBTTagByteArrayBuffered.readFrom(name, buffers.get(name), stream);
        } else if (type == 9) {
        	nbtbase = NBTTagListBuffered.readFrom(name, buffers, stream);
        } else {
        	nbtbase = NBTBase.createTag(type, name);
        	if (nbtbase instanceof NBTTagByte) {
        		((NBTTagByte) nbtbase).data = stream.readByte();
        	} else if (nbtbase instanceof NBTTagShort) {
        		((NBTTagShort) nbtbase).data = stream.readShort();
        	} else if (nbtbase instanceof NBTTagInt) {
        		((NBTTagInt) nbtbase).data = stream.readInt();
        	} else if (nbtbase instanceof NBTTagLong) {
        		((NBTTagLong) nbtbase).data = stream.readLong();
        	} else if (nbtbase instanceof NBTTagFloat) {
        		((NBTTagFloat) nbtbase).data = stream.readFloat();
        	} else if (nbtbase instanceof NBTTagDouble) {
        		((NBTTagDouble) nbtbase).data = stream.readDouble();
        	} else if (nbtbase instanceof NBTTagString) {
        		((NBTTagString) nbtbase).data = stream.readUTF();
        	}
        }
		return nbtbase;
	}


}
