package com.bergerkiller.bukkit.nolagg.sending;

import org.bukkit.block.BlockFace;

import com.bergerkiller.bukkit.nolagg.Util;

public class ChunkCoordIndexer {
	private static int[][][] indices = new int[8][Util.viewWidth][Util.viewWidth];
	static {
		new ChunkCoordIndexer(BlockFace.NORTH).generate();
		new ChunkCoordIndexer(BlockFace.NORTH_EAST).generate();
		new ChunkCoordIndexer(BlockFace.EAST).generate();
		new ChunkCoordIndexer(BlockFace.SOUTH_EAST).generate();
		new ChunkCoordIndexer(BlockFace.SOUTH).generate();
		new ChunkCoordIndexer(BlockFace.SOUTH_WEST).generate();
		new ChunkCoordIndexer(BlockFace.WEST).generate();
		new ChunkCoordIndexer(BlockFace.NORTH_WEST).generate();
	}
		
	public static int get(final int dx, final int dz, final BlockFace direction) {
		if (Math.abs(dx) > Util.view) return Integer.MAX_VALUE;
		if (Math.abs(dz) > Util.view) return Integer.MAX_VALUE;
		return indices[getFaceIndex(direction)][dx + Util.view][dz + Util.view];
	}
	
	private static int getFaceIndex(BlockFace face) {
		switch (face) {
		case NORTH : return 0;
		case NORTH_EAST : return 1;
		case EAST : return 2;
		case SOUTH_EAST : return 3;
		case SOUTH : return 4;
		case SOUTH_WEST : return 5;
		case WEST : return 6;
		case NORTH_WEST : return 7;
		default : return 0;
		}
	}
	
	private final BlockFace direction;
	private final int directionIndex;
	private int index = -1;
	private ChunkCoordIndexer(final BlockFace direction) {
		this.direction = direction;
		this.directionIndex = getFaceIndex(direction);
	}
	
	private static class MoveMod {
		private MoveMod(BlockFace direction, boolean right) {
			this.direction = direction;
			this.right = right;
		}
		
		public BlockFace direction;
		public boolean right;
		
		public void next(int dx, int dz, int limit) {
			if ((dx == limit || dx == -limit) && (dz == limit || dz == -limit)) {
				if (this.right) {
					if (direction == BlockFace.NORTH) {
						direction = BlockFace.EAST;
					} else if (direction == BlockFace.EAST) {
						direction = BlockFace.SOUTH;
					} else if (direction == BlockFace.SOUTH) {
						direction = BlockFace.WEST;
					} else if (direction == BlockFace.WEST) {
						direction = BlockFace.NORTH;
					}
				} else {
					if (direction == BlockFace.NORTH) {
						direction = BlockFace.WEST;
					} else if (direction == BlockFace.WEST) {
						direction = BlockFace.SOUTH;
					} else if (direction == BlockFace.SOUTH) {
						direction = BlockFace.EAST;
					} else if (direction == BlockFace.EAST) {
						direction = BlockFace.NORTH;
					}
				}
			}
		}
		
		public static MoveMod[] get(BlockFace direction) {
			MoveMod[] mods = new MoveMod[2];
			if (direction == BlockFace.NORTH) {
				mods[0] = new MoveMod(BlockFace.WEST, false);
				mods[1] = new MoveMod(BlockFace.EAST, true);
			} else if (direction == BlockFace.SOUTH) {
				mods[0] = new MoveMod(BlockFace.WEST, true);
				mods[1] = new MoveMod(BlockFace.EAST, false);
			} else if (direction == BlockFace.EAST) {
				mods[0] = new MoveMod(BlockFace.NORTH, false);
				mods[1] = new MoveMod(BlockFace.SOUTH, true);
			} else if (direction == BlockFace.WEST) {
				mods[0] = new MoveMod(BlockFace.NORTH, true);
				mods[1] = new MoveMod(BlockFace.SOUTH, false);
			} else if (direction == BlockFace.NORTH_EAST) {
				mods[0] = new MoveMod(BlockFace.WEST, false);
				mods[1] = new MoveMod(BlockFace.SOUTH, true);
			} else if (direction == BlockFace.SOUTH_EAST) {
				mods[0] = new MoveMod(BlockFace.WEST, true);
				mods[1] = new MoveMod(BlockFace.NORTH, false);
			} else if (direction == BlockFace.SOUTH_WEST) {
				mods[0] = new MoveMod(BlockFace.NORTH, true);
				mods[1] = new MoveMod(BlockFace.EAST, false);
			} else if (direction == BlockFace.NORTH_WEST) {
				mods[0] = new MoveMod(BlockFace.SOUTH, false);
				mods[1] = new MoveMod(BlockFace.EAST, true);
			}
			return mods;
		}	
	}
	
	private void generate(final int dx, final int dz) {
		indices[this.directionIndex][dx + Util.view][dz + Util.view] = this.index++;
	}
	
	private void generateLayer(final int layer, final double factor) {
		int count = (int) (layer * factor) + 1;
		//get modifiers from direction
		MoveMod[] mods = MoveMod.get(direction);
		//Get the chunk to start at
		int startx = direction.getModX() * layer;
		int startz = direction.getModZ() * layer;
		//Send starter chunk
		this.generate(startx, startz);
		//Peel
		int x1 = startx;
		int z1 = startz;
		int x2 = startx;
		int z2 = startz;
		while (--count > 0) {
			//offset the chunks
			x1 += mods[0].direction.getModX();
			z1 += mods[0].direction.getModZ();
			x2 += mods[1].direction.getModX();
			z2 += mods[1].direction.getModZ();
			//mod update
			mods[0].next(x1, z1, layer);
			mods[1].next(x2, z2, layer);
			//got till the end?
			this.generate(x1, z1);
			if (x1 == x2 && z1 == z2) {
				return;
			} else {
				this.generate(x2, z2);
			}
		}
	}
	
	private void generate() {
		//main chunk
		this.generate(0, 0);
		
		final int threshold1 = 2; //to this layer full layers are sent, after half
		final int threshold2 = 4; //at this layer less than half are sent
		
		//full layers
		for (int layer = 1; layer < threshold1; layer++) {
			this.generateLayer(layer, 4);
		}
		
		//half layers
		for (int layer = threshold1; layer <= threshold2; layer++) {
			this.generateLayer(layer, 2);
		}
		
		//less than half layers
		for (int layer = threshold2; layer <= Util.view; layer++) {
			this.generateLayer(layer, 1.5);
		}
		
		//end with only full layers
		for (int layer = threshold1; layer <= Util.view; layer++) {
			this.generateLayer(layer, 4);
		}
	}
	
	
}
